@@ -3,6 +3,7 @@
 # llvm-toolchain-snapshot-3.2_3.2repack.orig-clang.tar.bz2
 # llvm-toolchain-snapshot-3.2_3.2repack.orig-clang-extra.tar.bz2
 # llvm-toolchain-snapshot-3.2_3.2repack.orig-compiler-rt.tar.bz2
+# llvm-toolchain-snapshot-3.2_3.2repack.orig-lld.tar.bz2
 # llvm-toolchain-snapshot-3.2_3.2repack.orig-lldb.tar.bz2
 # llvm-toolchain-snapshot-3.2_3.2repack.orig-polly.tar.bz2
 # llvm-toolchain-snapshot-3.2_3.2repack.orig.tar.bz2
@@ -14,8 +15,8 @@ set -e
 # sh 3.4/debian/orig-tar.sh RELEASE_34 rc1
 
 SVN_BASE_URL=http://llvm.org/svn/llvm-project/
-MAJOR_VERSION=3.5
-CURRENT_VERSION=3.5.2 # Should be changed to 3.5.1 later
+MAJOR_VERSION=3.9
+CURRENT_VERSION=3.9 # Should be changed to 3.5.1 later
 
 if test -n "$1"; then
 # http://llvm.org/svn/llvm-project/{cfe,llvm,compiler-rt,...}/branches/google/stable/
@@ -48,7 +49,7 @@ get_svn_url() {
 }
 
 get_higher_revision() {
-    PROJECTS="llvm cfe compiler-rt polly lldb clang-tools-extra"
+    PROJECTS="llvm cfe compiler-rt polly lld lldb clang-tools-extra"
     REVISION_MAX=0
     for f in $PROJECTS; do
         REVISION=$(LANG=C svn info $(get_svn_url $f $BRANCH $TAG)|grep "^Last Changed Rev:"|awk '{print $4}')
@@ -59,7 +60,39 @@ get_higher_revision() {
     echo $REVISION_MAX
 }
 
+SVN_ARCHIVES=svn-archives
 
+checkout_sources() {
+    PROJECT=$1
+    URL=$2
+    TARGET=$3
+    BRANCH=$4
+    if test -n "$BRANCH"; then
+        REVISION=$5
+    fi
+    echo "$PROJECT / $URL / $BRANCH / $TARGET / $REVISION"
+
+    cd $SVN_ARCHIVES/
+    DEST=$PROJECT-$BRANCH
+    if test -d $DEST; then
+        cd $DEST
+        if test -n "$BRANCH"; then
+            svn up
+        else
+            svn up -r $REVISION
+        fi
+        cd ..
+    else
+        if test -n "$BRANCH"; then
+            svn co $URL $DEST
+        else
+            svn co -r $REVISION $URL $DEST
+        fi
+    fi
+    rm -rf ../$TARGET
+    rsync -r --exclude=.svn $DEST/ ../$TARGET
+    cd ..
+}
 
 if test -n "$BRANCH"; then
     REVISION=$(get_higher_revision)
@@ -73,7 +106,11 @@ fi
 
 if test -n "$RCRELEASE"; then
 #    VERSION=$MAJOR_VERSION"+"$REVISION # WAS TAG
-    VERSION=$CURRENT_VERSION"~+"$TAG
+    if test "$TAG" = "final"; then
+	VERSION=$MAJOR_VERSION
+    else
+        VERSION=$MAJOR_VERSION"~+"$TAG
+    fi
     FULL_VERSION="llvm-toolchain-"$MAJOR_VERSION"_"$VERSION
 else
     VERSION=$CURRENT_VERSION"~svn"$REVISION
@@ -84,42 +121,55 @@ else
     fi
 fi
 
+mkdir -p $SVN_ARCHIVES
+
 # LLVM
 LLVM_TARGET=$FULL_VERSION
-$SVN_CMD $(get_svn_url llvm $BRANCH $TAG) $LLVM_TARGET
+checkout_sources llvm $(get_svn_url llvm $BRANCH $TAG) $LLVM_TARGET "$BRANCH" $REVISION
 tar jcvf $FULL_VERSION.orig.tar.bz2 $LLVM_TARGET
 rm -rf $LLVM_TARGET
 
 
 # Clang
 CLANG_TARGET=clang_$VERSION
-$SVN_CMD $(get_svn_url cfe $BRANCH $TAG) $CLANG_TARGET
+checkout_sources clang $(get_svn_url cfe $BRANCH $TAG) $CLANG_TARGET "$BRANCH" $REVISION
 tar jcvf $FULL_VERSION.orig-clang.tar.bz2 $CLANG_TARGET
 rm -rf $CLANG_TARGET
 
 
 # Clang extra
 CLANG_TARGET=clang-tools-extra_$VERSION
-$SVN_CMD $(get_svn_url clang-tools-extra $BRANCH $TAG) $CLANG_TARGET
+checkout_sources clang-tools-extra $(get_svn_url clang-tools-extra $BRANCH $TAG) $CLANG_TARGET "$BRANCH" $REVISION
 tar jcvf $FULL_VERSION.orig-clang-tools-extra.tar.bz2 $CLANG_TARGET
 rm -rf $CLANG_TARGET
 
 # Compiler-rt
 COMPILER_RT_TARGET=compiler-rt_$VERSION
-$SVN_CMD $(get_svn_url compiler-rt $BRANCH $TAG) $COMPILER_RT_TARGET
+checkout_sources compiler-rt $(get_svn_url compiler-rt $BRANCH $TAG) $COMPILER_RT_TARGET "$BRANCH" $REVISION
+#$SVN_CMD $(get_svn_url compiler-rt $BRANCH $TAG) $COMPILER_RT_TARGET
 tar jcvf $FULL_VERSION.orig-compiler-rt.tar.bz2 $COMPILER_RT_TARGET
 rm -rf $COMPILER_RT_TARGET
 
 # Polly
 POLLY_TARGET=polly_$VERSION
-$SVN_CMD $(get_svn_url polly $BRANCH $TAG) $POLLY_TARGET
-rm -rf $POLLY_TARGET/www
+checkout_sources polly $(get_svn_url polly $BRANCH $TAG) $POLLY_TARGET "$BRANCH" $REVISION
+#$SVN_CMD $(get_svn_url polly $BRANCH $TAG) $POLLY_TARGET
+rm -rf $POLLY_TARGET/www $POLLY_TARGET/autoconf/config.sub $POLLY_TARGET/autoconf/config.guess
 tar jcvf $FULL_VERSION.orig-polly.tar.bz2 $POLLY_TARGET
 rm -rf $POLLY_TARGET
 
+# LLD
+LLD_TARGET=lld_$VERSION
+checkout_sources lld $(get_svn_url lld $BRANCH $TAG) $LLD_TARGET "$BRANCH" $REVISION
+#$SVN_CMD $(get_svn_url lld $BRANCH $TAG) $LLD_TARGET
+rm -rf $LLD_TARGET/www/
+tar jcvf $FULL_VERSION.orig-lld.tar.bz2 $LLD_TARGET
+rm -rf $LLD_TARGET
+
 # LLDB
 LLDB_TARGET=lldb_$VERSION
-$SVN_CMD $(get_svn_url lldb $BRANCH $TAG) $LLDB_TARGET
+checkout_sources lldb $(get_svn_url lldb $BRANCH $TAG) $LLDB_TARGET "$BRANCH" $REVISION
+#$SVN_CMD $(get_svn_url lldb $BRANCH $TAG) $LLDB_TARGET
 rm -rf $LLDB_TARGET/www/
 tar jcvf $FULL_VERSION.orig-lldb.tar.bz2 $LLDB_TARGET
 rm -rf $LLDB_TARGET
